@@ -182,7 +182,7 @@ static void at_registration_status(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = nd;
@@ -349,7 +349,7 @@ static void at_current_operator(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -408,8 +408,7 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	DBG("Got %d elements", num);
 
 	list = g_try_new0(struct ofono_network_operator, num);
-
-	if (!list) {
+	if (list == NULL) {
 		CALLBACK_WITH_FAILURE(cb, 0, NULL, cbd->data);
 		return;
 	}
@@ -492,7 +491,7 @@ static void at_list_operators(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=?", cops_prefix,
@@ -522,7 +521,7 @@ static void at_register_auto(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=0", none_prefix,
@@ -543,7 +542,7 @@ static void at_register_manual(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+COPS=1,2,\"%s%s\"", mcc, mnc);
@@ -564,7 +563,7 @@ static void at_deregister(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=2", none_prefix,
@@ -631,6 +630,23 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 				at_util_convert_signal_strength(strength));
 }
 
+static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
+{
+	//struct ofono_netreg *netreg = user_data;
+	const char *label;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XHOMEZR:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &label))
+		return;
+
+	ofono_info("Home zone: %s", label);
+}
+
 static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -681,6 +697,28 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 }
 
 static void ctzv_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	const char *tz;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CTZV:"))
+		return;
+
+	if (!g_at_result_iter_next_unquoted_string(&iter, &tz))
+		return;
+
+	DBG("tz %s", tz);
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
+static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
@@ -713,7 +751,7 @@ static void ctzv_notify(GAtResult *result, gpointer user_data)
 	nd->time.year = 2000 + year;
 }
 
-static void ctzdst_notify(GAtResult *result, gpointer user_data)
+static void ifx_ctzdst_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
@@ -827,7 +865,7 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = nd;
@@ -852,6 +890,55 @@ error:
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
 
+static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	int year, mon, mday, hour, min, sec;
+	const char *tz, *time, *timestamp;
+	GAtResultIter iter;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "*ETZV:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_string(&iter, &tz) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_string(&iter, &time) == FALSE)
+		time = NULL;
+
+	if (g_at_result_iter_next_string(&iter, &timestamp) == FALSE)
+		timestamp = NULL;
+
+	DBG("tz %s time %s timestamp %s", tz, time, timestamp);
+
+	if (time == NULL) {
+		year = -1;
+		mon = -1;
+		mday = -1;
+		hour = -1;
+		min = -1;
+		sec = -1;
+	} else {
+		if (sscanf(time, "%u/%u/%u,%u:%u:%u", &year, &mon, &mday,
+						&hour, &min, &sec) != 6)
+		return;
+	}
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	nd->time.sec = sec;
+	nd->time.min = min;
+	nd->time.hour = hour;
+	nd->time.mday = mday;
+	nd->time.mon = mon;
+	nd->time.year = year;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
+}
+
 static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -961,7 +1048,7 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 	switch (nd->vendor) {
 	case OFONO_VENDOR_OPTION_HSO:
 		tq = g_new0(struct tech_query, 1);
-		if (!tq)
+		if (tq == NULL)
 			break;
 
 		tq->status = status;
@@ -1049,7 +1136,6 @@ error:
 	ofono_netreg_remove(netreg);
 }
 
-
 static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -1084,21 +1170,36 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 				NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT_OSQI?", none_prefix,
 				NULL, NULL, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "+CTZV:", ctzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
+						NULL, NULL, NULL);
 		break;
 	case OFONO_VENDOR_MBM:
+		/* Enable network registration updates */
 		g_at_chat_send(nd->chat, "AT*E2REG=1", none_prefix,
-					NULL, NULL, NULL);
+						NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT*EREG=2", none_prefix,
-					NULL, NULL, NULL);
+						NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT*EPSB=1", none_prefix,
-					NULL, NULL, NULL);
+						NULL, NULL, NULL);
 
+		/* Register for network technology updates */
 		g_at_chat_send(nd->chat, "AT*ERINFO=1", none_prefix,
-					NULL, NULL, NULL);
+						NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "*ERINFO:", mbm_erinfo_notify,
-					FALSE, netreg, NULL);
+						FALSE, netreg, NULL);
+
+		/* Register for network time update reports */
+		g_at_chat_register(nd->chat, "*ETZV:", mbm_etzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT*ETZR=2", none_prefix,
+						NULL, NULL, NULL);
+
 		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
-				cind_support_cb, netreg, NULL);
+					cind_support_cb, netreg, NULL);
 		return;
 	case OFONO_VENDOR_NOVATEL:
 		/*
@@ -1116,15 +1217,21 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	case OFONO_VENDOR_IFX:
 		/* Register for specific signal strength reports */
 		g_at_chat_register(nd->chat, "+XCIEV:", ifx_xciev_notify,
-					FALSE, netreg, NULL);
+						FALSE, netreg, NULL);
 		g_at_chat_send(nd->chat, "AT+XMER=1", none_prefix,
 						NULL, NULL, NULL);
 
+		/* Register for home zone reports */
+		g_at_chat_register(nd->chat, "+XHOMEZR:", ifx_xhomezr_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+XHOMEZR=1", none_prefix,
+						NULL, NULL, NULL);
+
 		/* Register for network time update reports */
-		g_at_chat_register(nd->chat, "+CTZV:", ctzv_notify,
-					FALSE, netreg, NULL);
-		g_at_chat_register(nd->chat, "+CTZDST:", ctzdst_notify,
-					FALSE, netreg, NULL);
+		g_at_chat_register(nd->chat, "+CTZV:", ifx_ctzv_notify,
+						FALSE, netreg, NULL);
+		g_at_chat_register(nd->chat, "+CTZDST:", ifx_ctzdst_notify,
+						FALSE, netreg, NULL);
 		g_at_chat_send(nd->chat, "AT+CTZR=1", none_prefix,
 						NULL, NULL, NULL);
 		break;
