@@ -236,7 +236,7 @@ static void hfp_registration_status(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -259,7 +259,7 @@ static void hfp_current_operator(struct ofono_netreg *netreg,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
@@ -284,7 +284,7 @@ static void hfp_signal_strength(struct ofono_netreg *netreg,
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = netreg;
