@@ -55,6 +55,13 @@ enum clip_status {
 	CLIP_STATUS_UNKNOWN
 };
 
+/* 27.007 Section 7.30 */
+enum cnap_status {
+	CNAP_STATUS_NOT_PROVISIONED = 0,
+	CNAP_STATUS_PROVISIONED,
+	CNAP_STATUS_UNKNOWN
+};
+
 /* 27.007 Section 7.8 */
 enum colp_status {
 	COLP_STATUS_NOT_PROVISIONED = 0,
@@ -71,6 +78,7 @@ enum colr_status {
 
 enum call_setting_type {
 	CALL_SETTING_TYPE_CLIP = 0,
+	CALL_SETTING_TYPE_CNAP,
 	CALL_SETTING_TYPE_COLP,
 	CALL_SETTING_TYPE_COLR,
 	CALL_SETTING_TYPE_CLIR,
@@ -81,6 +89,7 @@ struct ofono_call_settings {
 	int clir;
 	int colr;
 	int clip;
+	int cnap;
 	int colp;
 	int clir_setting;
 	int cw;
@@ -103,9 +112,21 @@ static const char *clip_status_to_string(int status)
 		return "disabled";
 	case CLIP_STATUS_PROVISIONED:
 		return "enabled";
-	default:
-		return "unknown";
 	}
+
+	return "unknown";
+}
+
+static const char *cnap_status_to_string(int status)
+{
+	switch (status) {
+	case CNAP_STATUS_NOT_PROVISIONED:
+		return "disabled";
+	case CNAP_STATUS_PROVISIONED:
+		return "enabled";
+	}
+
+	return "unknown";
 }
 
 static const char *colp_status_to_string(int status)
@@ -115,9 +136,9 @@ static const char *colp_status_to_string(int status)
 		return "disabled";
 	case COLP_STATUS_PROVISIONED:
 		return "enabled";
-	default:
-		return "unknown";
 	}
+
+	return "unknown";
 }
 
 static const char *colr_status_to_string(int status)
@@ -127,9 +148,9 @@ static const char *colr_status_to_string(int status)
 		return "disabled";
 	case COLR_STATUS_PROVISIONED:
 		return "enabled";
-	default:
-		return "unknown";
 	}
+
+	return "unknown";
 }
 
 static const char *hide_callerid_to_string(int status)
@@ -141,9 +162,9 @@ static const char *hide_callerid_to_string(int status)
 		return "enabled";
 	case OFONO_CLIR_OPTION_SUPPRESSION:
 		return "disabled";
-	default:
-		return "default";
 	}
+
+	return "default";
 }
 
 static const char *clir_status_to_string(int status)
@@ -157,9 +178,9 @@ static const char *clir_status_to_string(int status)
 		return "on";
 	case CLIR_STATUS_TEMPORARY_ALLOWED:
 		return "off";
-	default:
-		return "unknown";
 	}
+
+	return "unknown";
 }
 
 static void set_clir_network(struct ofono_call_settings *cs, int clir)
@@ -228,6 +249,28 @@ static void set_clip(struct ofono_call_settings *cs, int clip)
 						DBUS_TYPE_STRING, &str);
 }
 
+static void set_cnap(struct ofono_call_settings *cs, int cnap)
+{
+	DBusConnection *conn;
+	const char *path;
+	const char *str;
+
+	if (cs->cnap == cnap)
+		return;
+
+	cs->cnap = cnap;
+
+	conn = ofono_dbus_get_connection();
+	path = __ofono_atom_get_path(cs->atom);
+
+	str = cnap_status_to_string(cnap);
+
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_CALL_SETTINGS_INTERFACE,
+						"CallingNamePresentation",
+						DBUS_TYPE_STRING, &str);
+}
+
 static void set_colp(struct ofono_call_settings *cs, int colp)
 {
 	DBusConnection *conn;
@@ -246,7 +289,7 @@ static void set_colp(struct ofono_call_settings *cs, int colp)
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_SETTINGS_INTERFACE,
-						"CalledLinePresentation",
+						"ConnectedLinePresentation",
 						DBUS_TYPE_STRING, &str);
 }
 
@@ -268,7 +311,7 @@ static void set_colr(struct ofono_call_settings *cs, int colr)
 
 	ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_SETTINGS_INTERFACE,
-						"CalledLineRestriction",
+						"ConnectedLineRestriction",
 						DBUS_TYPE_STRING, &str);
 }
 
@@ -412,7 +455,7 @@ static gboolean cw_ss_control(int type,
 	int cls = BEARER_CLASS_SS_DEFAULT;
 	DBusMessage *reply;
 
-	if (!cs)
+	if (cs == NULL)
 		return FALSE;
 
 	if (strcmp(sc, "43"))
@@ -426,8 +469,8 @@ static gboolean cw_ss_control(int type,
 	if (strlen(sib) || strlen(sib) || strlen(dn))
 		goto bad_format;
 
-	if ((type == SS_CONTROL_TYPE_QUERY && !cs->driver->cw_query) ||
-		(type != SS_CONTROL_TYPE_QUERY && !cs->driver->cw_set)) {
+	if ((type == SS_CONTROL_TYPE_QUERY && cs->driver->cw_query == NULL) ||
+		(type != SS_CONTROL_TYPE_QUERY && cs->driver->cw_set == NULL)) {
 		reply = __ofono_error_not_implemented(msg);
 		goto error;
 	}
@@ -520,7 +563,7 @@ static void generate_ss_query_reply(struct ofono_call_settings *cs,
 	__ofono_dbus_pending_reply(&cs->pending, reply);
 }
 
-static void clip_colp_colr_ss_query_cb(const struct ofono_error *error,
+static void clip_cnap_colp_colr_ss_query_cb(const struct ofono_error *error,
 					int status, void *data)
 {
 	struct ofono_call_settings *cs = data;
@@ -542,29 +585,36 @@ static void clip_colp_colr_ss_query_cb(const struct ofono_error *error,
 		context = "CallingLinePresentation";
 		break;
 
+	case CALL_SETTING_TYPE_CNAP:
+		set_cnap(cs, status);
+		value = cnap_status_to_string(status);
+		context = "CallingNamePresentation";
+		break;
+
+
 	case CALL_SETTING_TYPE_COLP:
 		set_colp(cs, status);
 		value = colp_status_to_string(status);
-		context = "CalledLinePresentation";
+		context = "ConnectedLinePresentation";
 		break;
 
 	case CALL_SETTING_TYPE_COLR:
 		set_colr(cs, status);
 		value = colr_status_to_string(status);
-		context = "CalledLineRestriction";
+		context = "ConnectedLineRestriction";
 		break;
 
 	default:
 		__ofono_dbus_pending_reply(&cs->pending,
 				__ofono_error_failed(cs->pending));
-		ofono_error("Unknown type during COLR/COLP/CLIP ss");
+		ofono_error("Unknown type during COLR/COLP/CLIP/CNAP ss");
 		return;
 	};
 
 	generate_ss_query_reply(cs, context, value);
 }
 
-static gboolean clip_colp_colr_ss(int type,
+static gboolean clip_cnap_colp_colr_ss(int type,
 				const char *sc, const char *sia,
 				const char *sib, const char *sic,
 				const char *dn, DBusMessage *msg, void *data)
@@ -574,7 +624,7 @@ static gboolean clip_colp_colr_ss(int type,
 	void (*query_op)(struct ofono_call_settings *cs,
 				ofono_call_settings_status_cb_t cb, void *data);
 
-	if (!cs)
+	if (cs == NULL)
 		return FALSE;
 
 	if (__ofono_call_settings_is_busy(cs)) {
@@ -587,6 +637,9 @@ static gboolean clip_colp_colr_ss(int type,
 	if (!strcmp(sc, "30")) {
 		cs->ss_setting = CALL_SETTING_TYPE_CLIP;
 		query_op = cs->driver->clip_query;
+	} else if (!strcmp(sc, "300")) {
+		cs->ss_setting = CALL_SETTING_TYPE_CNAP;
+		query_op = cs->driver->cnap_query;
 	} else if (!strcmp(sc, "76")) {
 		cs->ss_setting = CALL_SETTING_TYPE_COLP;
 		query_op = cs->driver->colp_query;
@@ -605,18 +658,18 @@ static gboolean clip_colp_colr_ss(int type,
 		return TRUE;
 	}
 
-	if (!query_op) {
+	if (query_op == NULL) {
 		DBusMessage *reply = __ofono_error_not_implemented(msg);
 		g_dbus_send_message(conn, reply);
 
 		return TRUE;
 	}
 
-	DBG("Received CLIP/COLR/COLP query ss control");
+	DBG("Received CLIP/CNAP/COLR/COLP query ss control");
 
 	cs->pending = dbus_message_ref(msg);
 
-	query_op(cs, clip_colp_colr_ss_query_cb, cs);
+	query_op(cs, clip_cnap_colp_colr_ss_query_cb, cs);
 
 	return TRUE;
 }
@@ -694,7 +747,7 @@ static gboolean clir_ss_control(int type,
 	struct ofono_call_settings *cs = data;
 	DBusConnection *conn = ofono_dbus_get_connection();
 
-	if (!cs)
+	if (cs == NULL)
 		return FALSE;
 
 	if (strcmp(sc, "31"))
@@ -726,7 +779,7 @@ static gboolean clir_ss_control(int type,
 		return TRUE;
 	}
 
-	if (type != SS_CONTROL_TYPE_QUERY && !cs->driver->clir_set) {
+	if (type != SS_CONTROL_TYPE_QUERY && cs->driver->clir_set == NULL) {
 		DBusMessage *reply = __ofono_error_not_implemented(msg);
 		g_dbus_send_message(conn, reply);
 
@@ -762,15 +815,19 @@ static gboolean clir_ss_control(int type,
 
 static void cs_register_ss_controls(struct ofono_call_settings *cs)
 {
-	__ofono_ussd_ssc_register(cs->ussd, "30", clip_colp_colr_ss, cs, NULL);
+	__ofono_ussd_ssc_register(cs->ussd, "30", clip_cnap_colp_colr_ss,
+								cs, NULL);
 	__ofono_ussd_ssc_register(cs->ussd, "31", clir_ss_control, cs, NULL);
-	__ofono_ussd_ssc_register(cs->ussd, "76", clip_colp_colr_ss, cs, NULL);
+	__ofono_ussd_ssc_register(cs->ussd, "76", clip_cnap_colp_colr_ss,
+								cs, NULL);
+	__ofono_ussd_ssc_register(cs->ussd, "300", clip_cnap_colp_colr_ss,
+								cs, NULL);
 
 	__ofono_ussd_ssc_register(cs->ussd, "43", cw_ss_control, cs, NULL);
 
 	if (cs->driver->colr_query)
 		__ofono_ussd_ssc_register(cs->ussd, "77",
-						clip_colp_colr_ss, cs, NULL);
+					clip_cnap_colp_colr_ss, cs, NULL);
 }
 
 static void cs_unregister_ss_controls(struct ofono_call_settings *cs)
@@ -778,6 +835,7 @@ static void cs_unregister_ss_controls(struct ofono_call_settings *cs)
 	__ofono_ussd_ssc_unregister(cs->ussd, "30");
 	__ofono_ussd_ssc_unregister(cs->ussd, "31");
 	__ofono_ussd_ssc_unregister(cs->ussd, "76");
+	__ofono_ussd_ssc_unregister(cs->ussd, "300");
 
 	__ofono_ussd_ssc_unregister(cs->ussd, "43");
 
@@ -799,8 +857,7 @@ static DBusMessage *generate_get_properties_reply(struct ofono_call_settings *cs
 	const char *str;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -813,12 +870,16 @@ static DBusMessage *generate_get_properties_reply(struct ofono_call_settings *cs
 	ofono_dbus_dict_append(&dict, "CallingLinePresentation",
 				DBUS_TYPE_STRING, &str);
 
+	str = cnap_status_to_string(cs->cnap);
+	ofono_dbus_dict_append(&dict, "CallingNamePresentation",
+				DBUS_TYPE_STRING, &str);
+
 	str = colp_status_to_string(cs->colp);
-	ofono_dbus_dict_append(&dict, "CalledLinePresentation",
+	ofono_dbus_dict_append(&dict, "ConnectedLinePresentation",
 				DBUS_TYPE_STRING, &str);
 
 	str = colr_status_to_string(cs->colr);
-	ofono_dbus_dict_append(&dict, "CalledLineRestriction",
+	ofono_dbus_dict_append(&dict, "ConnectedLineRestriction",
 				DBUS_TYPE_STRING, &str);
 
 	str = clir_status_to_string(cs->clir);
@@ -859,7 +920,7 @@ out:
 
 static void query_clir(struct ofono_call_settings *cs)
 {
-	if (!cs->driver->clir_query) {
+	if (cs->driver->clir_query == NULL) {
 		if (cs->pending) {
 			DBusMessage *reply =
 				generate_get_properties_reply(cs,
@@ -873,6 +934,27 @@ static void query_clir(struct ofono_call_settings *cs)
 	cs->driver->clir_query(cs, cs_clir_callback, cs);
 }
 
+static void cs_cnap_callback(const struct ofono_error *error,
+				int state, void *data)
+{
+	struct ofono_call_settings *cs = data;
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
+		set_cnap(cs, state);
+
+	query_clir(cs);
+}
+
+static void query_cnap(struct ofono_call_settings *cs)
+{
+	if (cs->driver->cnap_query == NULL) {
+		query_clir(cs);
+		return;
+	}
+
+	cs->driver->cnap_query(cs, cs_cnap_callback, cs);
+}
+
 static void cs_clip_callback(const struct ofono_error *error,
 				int state, void *data)
 {
@@ -881,12 +963,12 @@ static void cs_clip_callback(const struct ofono_error *error,
 	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
 		set_clip(cs, state);
 
-	query_clir(cs);
+	query_cnap(cs);
 }
 
 static void query_clip(struct ofono_call_settings *cs)
 {
-	if (!cs->driver->clip_query) {
+	if (cs->driver->clip_query == NULL) {
 		query_clir(cs);
 		return;
 	}
@@ -907,7 +989,7 @@ static void cs_colp_callback(const struct ofono_error *error,
 
 static void query_colp(struct ofono_call_settings *cs)
 {
-	if (!cs->driver->colp_query) {
+	if (cs->driver->colp_query == NULL) {
 		query_clip(cs);
 		return;
 	}
@@ -928,7 +1010,7 @@ static void cs_colr_callback(const struct ofono_error *error,
 
 static void query_colr(struct ofono_call_settings *cs)
 {
-	if (!cs->driver->colr_query) {
+	if (cs->driver->colr_query == NULL) {
 		query_colp(cs);
 		return;
 	}
@@ -949,7 +1031,7 @@ static void cs_cw_callback(const struct ofono_error *error, int status,
 
 static void query_cw(struct ofono_call_settings *cs)
 {
-	if (!cs->driver->cw_query) {
+	if (cs->driver->cw_query == NULL) {
 		query_colr(cs);
 		return;
 	}
@@ -1261,6 +1343,7 @@ struct ofono_call_settings *ofono_call_settings_create(struct ofono_modem *modem
 
 	/* Set all the settings to unknown state */
 	cs->clip = 2;
+	cs->cnap = 2;
 	cs->clir = 2;
 	cs->colp = 2;
 	cs->colr = 2;
